@@ -87,13 +87,17 @@ if (-f ${synthdir}/${rootname}_powerground) then
    source ${synthdir}/${rootname}_powerground
 endif
 
-# Prepend techdir to leffile unless leffile begins with "/"
-set abspath=`echo ${leffile} | cut -c1`
-if ( "${abspath}" == "/" ) then
-   set lefpath=${leffile}
-else
-   set lefpath=${techdir}/${leffile}
-endif
+# Prepend techdir to each leffile unless leffile begins with "/"
+set lefpath=""
+foreach f (${leffile})
+   set abspath=`echo ${f} | cut -c1`
+   if ( "${abspath}" == "/" ) then
+      set p=${leffile}
+   else
+      set p=${techdir}/${leffile}
+   endif
+   set lefpath="${lefpath} $p"
+end
 
 # Prepend techdir to techleffile unless techleffile begins with "/"
 set abspath=`echo ${techleffile} | cut -c1`
@@ -103,13 +107,17 @@ else
    set techlefpath=${techdir}/${techleffile}
 endif
 
-# Prepend techdir to spicefile unless spicefile begins with "/"
-set abspath=`echo ${spicefile} | cut -c1`
-if ( "${abspath}" == "/" ) then
-   set spicepath=${spicefile}
-else
-   set spicepath=${techdir}/${spicefile}
-endif
+# Prepend techdir to each spicefile unless spicefile begins with "/"
+set spicepath=""
+foreach f (${spicefile})
+   set abspath=`echo ${f} | cut -c1`
+   if ( "${abspath}" == "/" ) then
+      set p=${spicefile}
+   else
+      set p=${techdir}/${spicefile}
+   endif
+   set spicepath="${spicepath} $p"
+end
 
 if (!($?logdir)) then
    set logdir=${projectpath}/log
@@ -121,6 +129,10 @@ rm -f ${synthlog} >& /dev/null
 rm -f ${logdir}/sta.log >& /dev/null
 rm -f ${logdir}/route.log >& /dev/null
 rm -f ${logdir}/post_sta.log >& /dev/null
+rm -f ${logdir}/migrate.log >& /dev/null
+rm -f ${logdir}/drc.log >& /dev/null
+rm -f ${logdir}/lvs.log >& /dev/null
+rm -f ${logdir}/gdsii.log >& /dev/null
 touch ${synthlog}
 set date=`date`
 echo "Qflow placement logfile created on $date" > ${synthlog}
@@ -137,23 +149,95 @@ if ( ${errcond} == 1 ) then
    exit 1
 endif
 
+#---------------------------------------------------
+# Create .info file from qrouter
+#---------------------------------------------------
+
+cd ${layoutdir}
+
+# First prepare a simple .cfg file that can be used to point qrouter
+# to the LEF files when generating layer information using the "-i" option.
+# This also contains the scaling units used in the .cel and .def files.
+
+#------------------------------------------------------------------
+# Determine the version number and availability of scripting
+# in qrouter.
+#------------------------------------------------------------------
+
+set version=`${bindir}/qrouter -v 0 -h | tail -1`
+set major=`echo $version | cut -d. -f1`
+set minor=`echo $version | cut -d. -f2`
+set subv=`echo $version | cut -d. -f3`
+set scripting=`echo $version | cut -d. -f4`
+
+# Create the initial (bootstrap) configuration file
+
+if ( $scripting == "T" ) then
+   if ( "$techleffile" == "" ) then
+      echo "read_lef ${lefpath}" > ${rootname}.cfg
+   else
+      echo "read_lef ${techlefpath}" > ${rootname}.cfg
+   endif
+else
+   if ( "$techleffile" == "" ) then
+      echo "lef ${lefpath}" > ${rootname}.cfg
+   else
+      echo "lef ${techlefpath}" > ${rootname}.cfg
+   endif
+endif
+
+${bindir}/qrouter -i ${rootname}.info -c ${rootname}.cfg
+
+#---------------------------------------------------------------------
+# Spot check:  Did qrouter produce file ${rootname}.info?
+#---------------------------------------------------------------------
+
+if ( !( -f ${rootname}.info || \
+	( -f ${rootname}.info && -M ${rootname}.info < -M ${rootname}.pin ))) then
+   echo "qrouter (-i) failure:  No file ${rootname}.info." |& tee -a ${synthlog}
+   echo "Premature exit." |& tee -a ${synthlog}
+   echo "Synthesis flow stopped due to error condition." >> ${synthlog}
+   exit 1
+endif
+
+# Pull scale units from the .info file.  Default units are centimicrons.
+set units=`cat ${rootname}.info | grep units | cut -d' ' -f3`
+if ( "${units}" == "" ) then
+    set units=100
+endif
+
 #-------------------------------------------------------------------------
 # Create the .cel file for GrayWolf
 #-------------------------------------------------------------------------
 
 cd ${projectpath}
 
-echo "Running blif2cel to generate input files for graywolf" |& tee -a ${synthlog}
 if ( "$techleffile" == "" ) then
-    echo "blif2cel.tcl --blif ${synthdir}/${rootname}.blif --lef ${lefpath} --cel ${layoutdir}/${rootname}.cel" |& tee -a ${synthlog}
-    ${scriptdir}/blif2cel.tcl --blif ${synthdir}/${rootname}.blif \
-	--lef ${lefpath} --cel ${layoutdir}/${rootname}.cel >>& ${synthlog}
+    set lefoptions=""
+    set addsoptions=""
 else
-    echo "blif2cel.tcl --blif ${synthdir}/${rootname}.blif --lef ${techlefpath} --lef ${lefpath} --cel ${layoutdir}/${rootname}.cel" |& tee -a ${synthlog}
-    ${scriptdir}/blif2cel.tcl --blif ${synthdir}/${rootname}.blif \
-	--lef ${techlefpath} --lef ${lefpath} \
-	--cel ${layoutdir}/${rootname}.cel >>& ${synthlog}
+    set lefoptions="--lef ${techlefpath}"
+    set addsoptions="-techlef ${techlefpath}"
 endif
+set lefoptions="${lefoptions} --lef ${lefpath}"
+
+# Pass additional .lef files to blif2cel.tcl from the hard macros list
+
+if ( ${?hard_macros} ) then
+    foreach macro_path ( $hard_macros )
+	foreach file ( `ls ${sourcedir}/${macro_path}` )
+	    if ( ${file:e} == "lef" ) then
+		set lefoptions="${lefoptions} --hard-macro ${sourcedir}/${macro_path}/${file}"
+		set addsoptions="${addsoptions} -hardlef ${sourcedir}/${macro_path}/${file}"
+	    endif
+	end
+    end
+endif
+
+echo "Running blif2cel to generate input files for graywolf" |& tee -a ${synthlog}
+echo "blif2cel.tcl --blif ${synthdir}/${rootname}.blif ${lefoptions} --cel ${layoutdir}/${rootname}.cel" |& tee -a ${synthlog}
+${scriptdir}/blif2cel.tcl --blif ${synthdir}/${rootname}.blif \
+	${lefoptions} --cel ${layoutdir}/${rootname}.cel >>& ${synthlog}
 
 set errcond = $status
 if ( ${errcond} != 0 ) then
@@ -178,21 +262,101 @@ if ( !( -f ${layoutdir}/${rootname}.cel || \
    exit 1
 endif
 
+#---------------------------------------------------------------------
+# Move to the layout directory
+#---------------------------------------------------------------------
+
+cd ${layoutdir}
+
+#---------------------------------------------------------------------
+# Check if a .cel1 file exists and needs to be prepended to .cel
+# This file may contain hard macro definitions that assert blockages
+# to the placement.
+#---------------------------------------------------------------------
+
+if ( -f ${rootname}.cel1 ) then
+   echo "Preparing layout blockages from ${rootname}.cel1" |& tee -a ${synthlog}
+   mv ${rootname}.cel ${rootname}_tmp.cel
+   cp ${rootname}.cel1 ${rootname}.cel
+   cat ${rootname}_tmp.cel >> ${rootname}.cel
+   rm -f ${rootname}_tmp.cel
+else
+   echo -n "No ${rootname}.cel1 file found for project. . . " \
+		|& tee -a ${synthlog}
+   echo "no partial blockages to apply to layout." |& tee -a ${synthlog}
+endif
+
+
 #-------------------------------------------------------------------------
 # If placement option "initial_density" is set, run the decongest
 # script.  This will annotate the .cel file with fill cells to pad
 # out the area to the specified density.
 #-------------------------------------------------------------------------
 
-cd ${layoutdir}
+# Set value ${fillers} to be equal either to a single cell name if
+# only one of (decapcell, antennacell, fillcell) is defined, or a
+# comma-separated triplet.  If only one is defined, then "fillcell"
+# is set to that name for those scripts that only handle one kind of
+# fill cell.
+
+# Make sure all cell types are defined, but an empty string if not used.
+
+if ( ! ${?fillcell} ) then
+   set fillcell = ""
+endif
+if ( ! ${?decapcell} ) then
+   set decapcell = ""
+endif
+if ( ! ${?antennacell} ) then
+   set antennacell = ""
+endif
+
+if ("x$fillcell" != "x") then
+   if ("x$decapcell" != "x") then
+      if ("x$antennacell" != "x") then
+	 set fillers = "${fillcell},${decapcell},${antennacell}"
+      else
+	 set fillers = "${fillcell},${decapcell},"
+      endif
+   else if ("x$antennacell" != "x") then
+      set fillers = "${fillcell},,${antennacell}"
+   else
+      set fillers = "${fillcell}"
+   endif
+else
+   if ("x$decapcell" != "x") then
+      if ("x$antennacell" != "x") then
+	 set fillers = ",${decapcell},${antennacell}"
+      else
+	 set fillers = ",${decapcell},"
+      endif
+      set fillcell = "${decapcell}"
+   else if ("x$antennacell" != "x") then
+      set fillers = ",,${antennacell}"
+      set fillcell = "${antennacell}"
+   else
+      # There is no fill cell, which is likely to produce poor results.
+      echo "Warning:  No fill cell types are defined in the tech setup script."
+      echo "This is likely to produce poor layout and/or poor routing results."
+      set fillers = ""
+      set fillcell = ""
+   endif
+endif
 
 if ( ${?initial_density} ) then
    echo "Running decongest to set initial density of ${initial_density}" \
 		|& tee -a ${synthlog}
-   echo "decongest.tcl ${rootname} ${lefpath} ${fillcell} ${initial_density}" \
+   if ( ${?fill_ratios} ) then
+        echo "decongest.tcl ${rootname} ${lefpath} ${fillers} ${initial_density} ${fill_ratios} --units=${units}" \
 		|& tee -a ${synthlog}
-   ${scriptdir}/decongest.tcl ${rootname} ${lefpath} \
-		${fillcell} ${initial_density} |& tee -a ${synthlog}
+	${scriptdir}/decongest.tcl ${rootname} ${lefpath} \
+		${fillers} ${initial_density} ${fill_ratios} --units=${units} |& tee -a ${synthlog}
+   else
+        echo "decongest.tcl ${rootname} ${lefpath} ${fillers} ${initial_density} --units=${units}" \
+		|& tee -a ${synthlog}
+	${scriptdir}/decongest.tcl ${rootname} ${lefpath} \
+		${fillers} ${initial_density} --units=${units} |& tee -a ${synthlog}
+   endif
    set errcond = $status
    if ( ${errcond} != 0 ) then
 	 echo "decongest.tcl failed with exit status ${errcond}" |& tee -a ${synthlog}
@@ -277,56 +441,34 @@ if ( !( -f ${rootname}.pin || \
    exit 1
 endif
 
+#---------------------------------------------------
+# Remove blockage hardcells defined in .cel1
+#---------------------------------------------------
+
+# Remove any existing .obs file as place2def.tcl will append to it.
+rm -f ${rootname}.obs
+
+if ( (-f ${rootname}.cel1) && (-f ${scriptdir}/removeblocks.tcl) ) then
+
+      echo "Running removeblocks to remove partical blockage references" \
+		|& tee -a ${synthlog}
+      echo "removeblocks.tcl ${rootname}" \
+		|& tee -a ${synthlog}
+
+      ${scriptdir}/removeblocks.tcl ${rootname} >>& ${synthlog}
+      set errcond = $status
+      if ( ${errcond} != 0 ) then
+	 echo "removeblocks.tcl failed with exit status ${errcond}" |& tee -a ${synthlog}
+	 echo "Ignoring. . . this may cause errors downstream." |& tee -a ${synthlog}
+      endif
+endif
+
 #---------------------------------------------------
 # 2) Prepare DEF and .cfg files for qrouter
 #---------------------------------------------------
 
-# First prepare a simple .cfg file that can be used to point qrouter
-# to the LEF files when generating layer information using the "-i" option.
-
 if ($makedef == 1) then
 
-   #------------------------------------------------------------------
-   # Determine the version number and availability of scripting
-   # in qrouter.
-   #------------------------------------------------------------------
-
-   set version=`${bindir}/qrouter -v 0 -h | tail -1`
-   set major=`echo $version | cut -d. -f1`
-   set minor=`echo $version | cut -d. -f2`
-   set subv=`echo $version | cut -d. -f3`
-   set scripting=`echo $version | cut -d. -f4`
-
-   # Create the initial (bootstrap) configuration file
-
-   if ( $scripting == "T" ) then
-      if ( "$techleffile" == "" ) then
-         echo "read_lef ${lefpath}" > ${rootname}.cfg
-      else
-         echo "read_lef ${techlefpath}" > ${rootname}.cfg
-      endif
-   else
-      if ( "$techleffile" == "" ) then
-         echo "lef ${lefpath}" > ${rootname}.cfg
-      else
-         echo "lef ${techlefpath}" > ${rootname}.cfg
-      endif
-   endif
-
-   ${bindir}/qrouter -i ${rootname}.info -c ${rootname}.cfg
-
-   #---------------------------------------------------------------------
-   # Spot check:  Did qrouter produce file ${rootname}.info?
-   #---------------------------------------------------------------------
-
-   if ( !( -f ${rootname}.info || \
-	( -f ${rootname}.info && -M ${rootname}.info < -M ${rootname}.pin ))) then
-      echo "qrouter (-i) failure:  No file ${rootname}.info." |& tee -a ${synthlog}
-      echo "Premature exit." |& tee -a ${synthlog}
-      echo "Synthesis flow stopped due to error condition." >> ${synthlog}
-      exit 1
-   endif
-
    # Run getfillcell to determine which cell should be used for fill to
    # match the width specified for feedthroughs in the .par file.  If
    # nothing is returned by getfillcell, then either feedthroughs have
@@ -343,12 +485,31 @@ if ($makedef == 1) then
    endif
    echo "Using cell ${usefillcell} for fill" |& tee -a ${synthlog}
 
-   # Run place2def to turn the GrayWolf output into a DEF file
+   # Set up antenna pin option, if it is defined, then use user-defined
+   # options for place2def, if they are defined.
+
+   if ( !( ${?antennapin_in} )) then
+      set antenna_opt = ""
+   else
+      if ( !( ${?antennacell} )) then
+         set antenna_opt = ""
+      else
+	 if ( "x${antennacell}" == "x" ) then
+            set antenna_opt = ""
+	 else
+            set antenna_opt = "antennapin=${antennapin_in} antennacell=${antennacell}"
+	 endif
+      endif
+   endif
 
    if ( !( ${?place2def_options} )) then
-      set place2def_options = ""
+      set place2def_options = "$antenna_opt"
+   else
+      set place2def_options = "$antenna_opt $place2def_options"
    endif
 
+   # Run place2def to turn the GrayWolf output into a DEF file
+
    echo "Running place2def to translate graywolf output to DEF format." \
 		|& tee -a ${synthlog}
    if ( ${?route_layers} ) then
@@ -388,12 +549,14 @@ if ($makedef == 1) then
 
    if ( !(${?nospacers}) && (-f ${scriptdir}/addspacers.tcl) ) then
 
-      if ( !( ${?addspacers_options} )) then
-         set addspacers_options = ""
-      endif
+      # Fill will use just the fillcell for padding under power buses
+      # and on the edges (to do:  refine this to use other spacer types
+      # if the width options are more flexible).
 
-      if ( "$techleffile" != "" ) then
-         set addspacers_options = "-techlef ${techlefpath} ${addspacers_options}"
+      if ( !( ${?addspacers_options} )) then
+         set addspacers_options = "${addsoptions}"
+      else
+         set addspacers_options = "${addsoptions} ${addspacers_options}"
       endif
 
       echo "Running addspacers to generate power stripes and align cell right edge" \
@@ -413,8 +576,6 @@ if ($makedef == 1) then
 
       if ( -f ${rootname}_filled.def ) then
 	 mv ${rootname}_filled.def ${rootname}.def
-	 # Copy the .def file to a backup called "unroute"
-	 cp ${rootname}.def ${rootname}_unroute.def
       endif
 
       if ( -f ${rootname}.obsx ) then
@@ -422,11 +583,44 @@ if ($makedef == 1) then
          # overwrite the original.
 	 mv ${rootname}.obsx ${rootname}.obs
       endif
-   else
-      # Copy the .def file to a backup called "unroute"
-      cp ${rootname}.def ${rootname}_unroute.def
    endif
 
+   #---------------------------------------------------------------------
+   # Run pin position adjustment script to make sure that pins avoid the
+   # power buses and are actually close to their respective connections
+   # in the digital core, which is something that graywolf has serious
+   # problems ensuring.  Also puts pins on a double-pitch spacing to make
+   # it much easier for the router to reach them.
+   #---------------------------------------------------------------------
+
+   if ( -f ${scriptdir}/arrangepins.tcl ) then
+
+      if ( !( ${?arrangepins_options} )) then
+         set arrangepins_options = ""
+      endif
+
+      echo "Running arrangepins to adjust pin positions for optimal routing." \
+		|& tee -a ${synthlog}
+      echo "arrangepins.tcl ${arrangepins_options} ${rootname}" |& tee -a ${synthlog}
+      ${scriptdir}/arrangepins.tcl ${arrangepins_options} ${rootname} \
+		|& tee -a ${synthlog}
+
+      # Check if the _mod.def output file was generated, and if so, rename it
+      # back to plain .def.
+
+      if ( !( -f ${rootname}_mod.def || ( -f ${rootname}_mode.def && \
+		-M ${rootname}_mod.def < -M ${rootname}.def ))) then
+          echo "Error (ignoring):"
+	  echo "   arrangepins.tcl failed to generate file ${rootname}_mod.def."
+      else
+          mv ${rootname}_mod.def ${rootname}.def
+      endif
+
+   endif
+
+   # Copy the .def file to a backup called "unroute"
+   cp ${rootname}.def ${rootname}_unroute.def
+
    # If the user didn't specify a number of layers for routing as part of
    # the project variables, then the info file created by qrouter will have
    # as many lines as there are route layers defined in the technology LEF
@@ -452,6 +646,17 @@ if ($makedef == 1) then
          echo "read_lef ${techlefpath}" >> ${rootname}.cfg
       endif
       echo "read_lef ${lefpath}" >> ${rootname}.cfg
+
+      if ( ${?hard_macros} ) then
+          foreach macro_path ( $hard_macros )
+	      foreach file ( `ls ${sourcedir}/${macro_path}` )
+		  if ( ${file:e} == "lef" ) then
+		      echo "read_lef ${sourcedir}/${macro_path}/${file}" >> ${rootname}.cfg
+	          endif
+	      end
+	  end
+      endif
+
       echo "catch {layers ${route_layers}}" >> ${rootname}.cfg
       if ( ${?via_use} ) then
          echo "" >> ${rootname}.cfg
@@ -475,6 +680,15 @@ if ($makedef == 1) then
          echo "lef ${techlefpath}" >> ${rootname}.cfg
       endif
       echo "lef ${lefpath}" >> ${rootname}.cfg
+      if ( ${?hard_macros} ) then
+          foreach macro_path ( $hard_macros )
+	      foreach file ( `ls ${sourcedir}/${macro_path}` )
+		  if ( ${file:e} == "lef" ) then
+		      echo "lef ${sourcedir}/${macro_path}/${file}" >> ${rootname}.cfg
+	          endif
+	      end
+	  end
+      endif
       echo "num_layers ${route_layers}" >> ${rootname}.cfg
       if ( ${?via_pattern} ) then
          echo "" >> ${rootname}.cfg
@@ -503,6 +717,9 @@ if ($makedef == 1) then
    # Scripted version continues with the read-in of the DEF file
 
    if (${scripting} == "T") then
+      if ("x$antennacell" != "x") then
+	 echo "catch {qrouter::antenna init ${antennacell}}" >> ${rootname}.cfg
+      endif
       echo "read_def ${rootname}.def" >> ${rootname}.cfg
    endif
 
@@ -517,9 +734,12 @@ if ($makedef == 1) then
    else
       if (${scripting} == "T") then
 	 echo "qrouter::standard_route ${rootname}_route.def false" >> ${rootname}.cfg
-	 echo "qrouter::write_delays ${rootname}.rc" >> ${rootname}.cfg
-	 # Standard_route's automatic quit has been subverted in order
-	 # to write the delay file, so make sure that qrouter actually exits.
+	 # write_delays folded into standard_route in qrouter version 1.4.21.
+	 if (${major} == 1 && ${minor} == 4 && ${subv} < 21) then
+	    echo "qrouter::write_delays ${rootname}_route.rc" >> ${rootname}.cfg
+	 endif
+	 # Qrouter will drop into the interpreter on failure, so force a
+	 # quit command to make sure that qrouter actually exits.
 	 echo "quit" >> ${rootname}.cfg
       endif
    endif
@@ -564,6 +784,7 @@ if ($makedef == 1) then
       echo "Files:" |& tee -a ${synthlog}
       echo "   Verilog: ${synthdir}/${rootname}.rtl.v" |& tee -a ${synthlog}
       echo "   Verilog: ${synthdir}/${rootname}.rtlnopwr.v" |& tee -a ${synthlog}
+      echo "   Verilog: ${synthdir}/${rootname}.rtlbb.v" |& tee -a ${synthlog}
       echo "   Spice:   ${synthdir}/${rootname}.spc" |& tee -a ${synthlog}
       echo "" >> ${synthlog}
 
@@ -574,9 +795,10 @@ if ($makedef == 1) then
       # pre- and post-placement netlists.
       #------------------------------------------------------------------
 
-      echo "Copying ${rootname}.rtl.v and ${rootname}.rtlnopwr.v to backups"
+      echo "Copying ${rootname}.rtl.v, ${rootname}.rtlnopwr.v, and ${rootname}.rtlbb.v to backups"
       cp ${rootname}.rtl.v ${rootname}_synth.rtl.v
       cp ${rootname}.rtlnopwr.v ${rootname}_synth.rtlnopwr.v
+      cp ${rootname}.rtlbb.v ${rootname}_synth.rtlbb.v
 
       echo "Running blif2Verilog." |& tee -a ${synthlog}
       ${bindir}/blif2Verilog -c -v ${vddnet} -g ${gndnet} \
@@ -585,6 +807,9 @@ if ($makedef == 1) then
       ${bindir}/blif2Verilog -c -p -v ${vddnet} -g ${gndnet} \
 		${rootname}_anno.blif > ${rootname}.rtlnopwr.v
 
+      ${bindir}/blif2Verilog -c -b -p -n -v ${vddnet} -g ${gndnet} \
+		${rootname}_anno.blif > ${rootname}.rtlbb.v
+
       echo "Running blif2BSpice." |& tee -a ${synthlog}
       ${bindir}/blif2BSpice -i -p ${vddnet} -g ${gndnet} -l \
 		${spicepath} ${rootname}_anno.blif \
@@ -595,19 +820,25 @@ if ($makedef == 1) then
       #------------------------------------------------------------------
 
       if ( !( -f ${rootname}.rtl.v || ( -f ${rootname}.rtl.v && \
-		-M ${rootname}.rtl.v < -M ${rootname}.blif ))) then
+		-M ${rootname}.rtl.v < -M ${rootname}_anno.blif ))) then
 	 echo "blif2Verilog failure:  No file ${rootname}.rtl.v created." \
 		|& tee -a ${synthlog}
       endif
 
       if ( !( -f ${rootname}.rtlnopwr.v || ( -f ${rootname}.rtlnopwr.v && \
-		-M ${rootname}.rtlnopwr.v < -M ${rootname}.blif ))) then
+		-M ${rootname}.rtlnopwr.v < -M ${rootname}_anno.blif ))) then
 	 echo "blif2Verilog failure:  No file ${rootname}.rtlnopwr.v created." \
 		|& tee -a ${synthlog}
       endif
 
+      if ( !( -f ${rootname}.rtlbb.v || ( -f ${rootname}.rtlbb.v && \
+		-M ${rootname}.rtlbb.v < -M ${rootname}_anno.blif ))) then
+	 echo "blif2Verilog failure:  No file ${rootname}.rtlbb.v created." \
+		|& tee -a ${synthlog}
+      endif
+
       if ( !( -f ${rootname}.spc || ( -f ${rootname}.spc && \
-		-M ${rootname}.spc < -M ${rootname}.blif ))) then
+		-M ${rootname}.spc < -M ${rootname}_anno.blif ))) then
 	 echo "blif2BSpice failure:  No file ${rootname}.spc created." \
 		|& tee -a ${synthlog}
       endif
