@@ -40,7 +40,8 @@ else
    echo
    echo			$yosys_options	for yosys
    echo			$yosys_script	for yosys
-   echo			$nobuffers	to bypass ybuffer
+   echo			$nobuffers	to ignore output buffering
+   echo			$inbuffers	to force input buffering
    echo			$fanout_options	for blifFanout
    exit 1
 endif
@@ -75,6 +76,10 @@ rm -f ${logdir}/place.log >& /dev/null
 rm -f ${logdir}/sta.log >& /dev/null
 rm -f ${logdir}/route.log >& /dev/null
 rm -f ${logdir}/post_sta.log >& /dev/null
+rm -f ${logdir}/migrate.log >& /dev/null
+rm -f ${logdir}/drc.log >& /dev/null
+rm -f ${logdir}/lvs.log >& /dev/null
+rm -f ${logdir}/gdsii.log >& /dev/null
 touch ${synthlog}
 set date=`date`
 echo "Qflow synthesis logfile created on $date" > ${synthlog}
@@ -87,22 +92,29 @@ else
    set libertypath=${techdir}/${libertyfile}
 endif
 
-# Prepend techdir to spicefile unless spicefile begins with "/"
-set abspath=`echo ${spicefile} | cut -c1`
-if ( "${abspath}" == "/" ) then
-   set spicepath=${spicefile}
-else
-   set spicepath=${techdir}/${spicefile}
-endif
+# Prepend techdir to each spicefile unless spicefile begins with "/"
+set spicepath=""
+foreach f (${spicefile})
+   set abspath=`echo ${f} | cut -c1`
+   if ( "${abspath}" == "/" ) then
+      set p=${spicefile}
+   else
+      set p=${techdir}/${spicefile}
+   endif
+   set spicepath="${spicepath} $p"
+end
 
 # Prepend techdir to leffile unless leffile begins with "/"
-set abspath=`echo ${leffile} | cut -c1`
-if ( "${abspath}" == "/" ) then
-   set lefpath=${leffile}
-else
-   set lefpath=${techdir}/${leffile}
-endif
-
+set lefpath=""
+foreach f (${leffile})
+   set abspath=`echo ${f} | cut -c1`
+   if ( "${abspath}" == "/" ) then
+      set p=${leffile}
+   else
+      set p=${techdir}/${leffile}
+   endif
+   set lefpath="${lefpath} $p"
+end
 
 # Determine version of yosys
 set versionstring = `${bindir}/yosys -V | cut -d' ' -f2`
@@ -127,7 +139,6 @@ else
 
 endif
 
-
 # Check if "yosys_options" specifies a script to use for yosys.
 if ( ! ${?yosys_options} ) then
    set yosys_options = ""
@@ -138,111 +149,74 @@ set usescript = `echo ${yosys_options} | grep -- -s | wc -l`
 # Only generate yosys script if none is specified in the yosys options
 if ( ${usescript} == 0 ) then
 
-#---------------------------------------------------------------------
-# Determine hierarchy by running yosys with a simple script to check
-# hierarchy.  Add files until yosys no longer reports an error.
-# Any error not related to a missing source file causes the script
-# to rerun yosys and dump error information into the log file, and
-# exit.
-#---------------------------------------------------------------------
-
 cd ${sourcedir}
 
-set uniquedeplist = ""
-set yerrcnt = 2
+# Check for filelists variable overriding the default.  If it is set,
+# use the value as the filename to get all the verilog source files
+# used by the module.  If not, check for a file ${modulename}.fl and
+# if it exists, use it (this is the default).  If not, then do an
+# automatic search for files containing modules.
 
-while ($yerrcnt > 1)
-
-# Note:  While the use of read_liberty to allow structural verilog only
-# works in yosys 0.3.1 and newer, the following line works for the
-# purpose of querying the hierarchy in all versions.
-
-set svopt = ""
-if ($sourcename == "") then
-   set vext = "v"
-
-   if ( !( -f ${modulename}.${vext} )) then
-      set vext = "sv"
-      set svopt = "-sv"
-      if ( !( -f ${modulename}.${vext} )) then
-         echo "Error:  Verilog source file ${modulename}.v (or .sv) cannot be found!" \
-		|& tee -a ${synthlog}
-      else
-         set sourcename = ${modulename}.${vext}
-      endif
-   else
-      set sourcename = ${modulename}.${vext}
+if ( ! ($?source_file_list) ) then
+   # Check for ${modulename}.fl
+   if ( -f ${modulename}.fl ) then
+      set source_file_list = ${modulename}.fl
    endif
-else
-   if ( !( -f ${sourcename} )) then
-      echo "Error:  Verilog source file ${sourcename} cannot be found!" \
-		|& tee -a ${synthlog}
-   else
-      set vext = ${sourcename:e}
-      if ( ${vext} == "sv" ) then
+endif
+
+if ( $?source_file_list ) then
+
+   # Use file list
+
+   set uniquedeplist = ""
+   if ( $?is_system_verilog ) then
+      if ( "$is_system_verilog" == "1" ) then
 	 set svopt = "-sv"
+      else
+	 set svopt = ""
       endif
+   else
+      set svopt = ""
    endif
-endif
 
-if (${sourcename} == "") then
-   echo "Synthesis flow stopped due to error condition." >> ${synthlog}
-   exit 1
-endif
-
-cat > ${modulename}.ys << EOF
+   cat > ${modulename}.ys << EOF
 # Synthesis script for yosys created by qflow
 read_liberty -lib -ignore_miss_dir -setattr blackbox ${libertypath}
 EOF
 
-# Add all entries in variable "hard_macros".  "hard_macros" should be a
-# list of directories found in ${sourcedir}, each directory the module
-# name of a single module.  The directory contains several files for
-# each hard macro, including .lib, .lef, .spc, .v, and possibly others.
-
-if ( ${?hard_macros} ) then
-    foreach macro_path ( $hard_macros )
-        foreach file ( `ls ${sourcedir}/${macro_path}` )
+   if ( ${?hard_macros} ) then
+      foreach macro_path ( $hard_macros )
+         foreach file ( `ls ${sourcedir}/${macro_path}` )
 	    if ( ${file:e} == "lib" ) then
 		cat >> ${modulename}.ys << EOF
 read_liberty -lib -ignore_miss_dir -setattr blackbox ${sourcedir}/${macro_path}/${file}
 EOF
 		break
 	    endif
-	end
-    end
-endif
+	 end
+      end
+   endif
 
-cat >> ${modulename}.ys << EOF
-read_verilog ${svopt} ${sourcename}
-EOF
+   set lines=`cat $source_file_list`
+   set i=1
+   while ( $i <= $#lines )
+      echo "read_verilog ${svopt} $lines[$i]" >> ${modulename}.ys
+      echo "Adding $lines[$i]"
+      set uniquedeplist = "$uniquedeplist $lines[$i]"
+      @ i = $i + 1
+   end
 
-foreach subname ( $uniquedeplist )
-    if ( !( -f ${subname}.${vext} )) then
-	echo "Error:  Verilog source file ${subname}.${vext} cannot be found!" \
-			|& tee -a ${synthlog}
-    endif
-    echo "read_verilog ${svopt} ${subname}.${vext}" >> ${modulename}.ys
-end
-
-cat >> ${modulename}.ys << EOF
+   cat >> ${modulename}.ys << EOF
 # Hierarchy check
 hierarchy -check
 EOF
 
-# Note:  Remove backslashes and brackets to avoid problems with tcsh
-set yerrors = `eval ${bindir}/yosys -s ${modulename}.ys |& sed -e "/\\/s#\\#/#g" \
+   # Note:  Remove backslashes and brackets to avoid problems with tcsh
+   set yerrors = `eval ${bindir}/yosys -s ${modulename}.ys |& sed -e "/\\/s#\\#/#g" \
 		-e "/\[/s/\[//g" -e "/\]/s/\]//g" | grep ERROR`
-set yerrcnt = `echo $yerrors | wc -c`
+   set yerrcnt = `echo $yerrors | wc -c`
 
-if ($yerrcnt > 1) then
-   set yvalid = `echo $yerrors | grep "referenced in module" | wc -c`
-   # Check error message specific to a missing source file.
-   set ymissing = `echo $yerrors | grep "is not part of" | wc -c`
-   if (($ymissing > 1) && ($yvalid > 1)) then
-      set newdep = `echo $yerrors | cut -d " " -f 3 | cut -c3- | cut -d "'" -f 1`
-      set uniquedeplist = "${uniquedeplist} ${newdep}"
-   else
+   if ($yerrcnt > 1) then
       ${bindir}/yosys -s ${modulename}.ys >& ${synthlog}
       echo "Errors detected in verilog source, need to be corrected." \
 		|& tee -a ${synthlog}
@@ -250,10 +224,154 @@ if ($yerrcnt > 1) then
       echo "Synthesis flow stopped due to error condition." >> ${synthlog}
       exit 1
    endif
-endif
+else
 
-# end while ($yerrcnt > 1)
-end
+   #---------------------------------------------------------------------
+   # Determine hierarchy by running yosys with a simple script to check
+   # hierarchy.  Add files until yosys no longer reports an error.
+   # Any error not related to a missing source file causes the script
+   # to rerun yosys and dump error information into the log file, and
+   # exit.
+   #---------------------------------------------------------------------
+
+   set uniquedeplist = ""
+   set yerrcnt = 2
+
+   while ($yerrcnt > 1)
+
+   # Use is_system_verilog to declare that files with ".v" extension
+   # are system verilog or system verilog compatible.
+
+   if ( $?is_system_verilog ) then
+      if ( "$is_system_verilog" == "1" ) then
+         set svopt = "-sv"
+      else
+         set svopt = ""
+      endif
+   else
+      set svopt = ""
+   endif
+ 
+   # Note:  While the use of read_liberty to allow structural verilog only
+   # works in yosys 0.3.1 and newer, the following line works for the
+   # purpose of querying the hierarchy in all versions.
+
+   if ($sourcename == "") then
+      set vext = "v"
+
+      if ( !( -f ${modulename}.${vext} )) then
+         set vext = "sv"
+         set svopt = "-sv"
+         if ( !( -f ${modulename}.${vext} )) then
+            echo "Error:  Verilog source file ${modulename}.v (or .sv) cannot be found!" \
+		|& tee -a ${synthlog}
+            set svopt = ""
+         else
+            set sourcename = ${modulename}.${vext}
+         endif
+      else
+         set sourcename = ${modulename}.${vext}
+      endif
+   else
+      if ( !( -f ${sourcename} )) then
+         echo "Error:  Verilog source file ${sourcename} cannot be found!" \
+		|& tee -a ${synthlog}
+      else
+         set vext = ${sourcename:e}
+         if ( ${vext} == "sv" ) then
+	    set svopt = "-sv"
+         endif
+      endif
+   endif
+
+   if (${sourcename} == "") then
+      echo "Synthesis flow stopped due to error condition." >> ${synthlog}
+      exit 1
+   endif
+
+   cat > ${modulename}.ys << EOF
+# Synthesis script for yosys created by qflow
+read_liberty -lib -ignore_miss_dir -setattr blackbox ${libertypath}
+EOF
+
+   # Add all entries in variable "hard_macros".  "hard_macros" should be a
+   # list of directories found in ${sourcedir}, each directory the module
+   # name of a single module.  The directory contains several files for
+   # each hard macro, including .lib, .lef, .spc, .v, and possibly others.
+
+   if ( ${?hard_macros} ) then
+       foreach macro_path ( $hard_macros )
+           foreach file ( `ls ${sourcedir}/${macro_path}` )
+	       if ( ${file:e} == "lib" ) then
+		   cat >> ${modulename}.ys << EOF
+read_liberty -lib -ignore_miss_dir -setattr blackbox ${sourcedir}/${macro_path}/${file}
+EOF
+		   break
+	       endif
+	   end
+       end
+   endif
+
+   cat >> ${modulename}.ys << EOF
+read_verilog ${svopt} ${sourcename}
+EOF
+
+   if ( ! ( $?vext ) ) then
+      if ( "$svopt" == "" ) then
+	 set vext = "v"
+      else
+	 set vext = "sv"
+      endif
+   endif
+
+   foreach subname ( $uniquedeplist )
+       if ( ${subname:e} == "" ) then
+           if ( !( -f ${subname}.${vext} )) then
+	       echo "Error:  Verilog source file ${subname}.${vext} cannot be found!" \
+			|& tee -a ${synthlog}
+	   endif
+           echo "read_verilog ${svopt} ${subname}.${vext}" >> ${modulename}.ys
+       else
+           if ( !( -f ${subname} )) then
+	       echo "Error:  Verilog source file ${subname} cannot be found!" \
+			|& tee -a ${synthlog}
+	   endif
+           echo "read_verilog ${svopt} ${subname}" >> ${modulename}.ys
+       endif
+   end
+
+   cat >> ${modulename}.ys << EOF
+# Hierarchy check
+hierarchy -check
+EOF
+
+   # Note:  Remove backslashes and brackets to avoid problems with tcsh
+   set yerrors = `eval ${bindir}/yosys -s ${modulename}.ys |& sed -e "/\\/s#\\#/#g" \
+		-e "/\[/s/\[//g" -e "/\]/s/\]//g" | grep ERROR`
+   set yerrcnt = `echo $yerrors | wc -c`
+
+   if ($yerrcnt > 1) then
+      set yvalid = `echo $yerrors | grep "referenced in module" | wc -c`
+      # Check error message specific to a missing source file.
+      set ymissing = `echo $yerrors | grep "is not part of" | wc -c`
+      if (($ymissing > 1) && ($yvalid > 1)) then
+         set newdep = `echo $yerrors | cut -d " " -f 3 | cut -c3- | cut -d "'" -f 1`
+         set uniquedeplist = "${uniquedeplist} ${newdep}"
+      else
+         ${bindir}/yosys -s ${modulename}.ys >& ${synthlog}
+         echo "Errors detected in verilog source, need to be corrected." \
+		|& tee -a ${synthlog}
+         echo "See file ${synthlog} for error output."
+         echo "Synthesis flow stopped due to error condition." >> ${synthlog}
+         exit 1
+      endif
+   endif
+
+   # end while ($yerrcnt > 1)
+   end
+
+# end if (! ($?source_file_list) )
+endif
 
 #---------------------------------------------------------------------
 # Generate the main yosys script
@@ -293,12 +411,31 @@ EOF
     end
 endif
 
-cat >> ${modulename}.ys << EOF
+# NOTE:  Using source_file_list, all files go into $uniquedeplist;  there
+# is no additional ${sourcename} file.
+
+if ( ! ( $?source_file_list )) then
+   if ( ${sourcename} != "" ) then
+      cat >> ${modulename}.ys << EOF
 read_verilog ${svopt} ${sourcename}
 EOF
+   endif
+endif
+
+if ( ! ( $?vext ) ) then
+   if ( "$svopt" == "" ) then
+      set vext = "v"
+   else
+      set vext = "sv"
+   endif
+endif
 
 foreach subname ( $uniquedeplist )
-    echo "read_verilog ${svopt} ${subname}.${vext}" >> ${modulename}.ys
+    if ( ${subname:e} == "" ) then
+       echo "read_verilog ${svopt} ${subname}.${vext}" >> ${modulename}.ys
+    else
+       echo "read_verilog ${svopt} ${subname}" >> ${modulename}.ys
+    endif
 end
 
 # Will not support yosys 0.0.x syntax; flag a warning instead
@@ -408,6 +545,12 @@ if ( ${major} > 0 || ${minor} > 1 ) then
        cat >> ${modulename}.ys << EOF
 # Output buffering
 iopadmap -outpad ${bufcell} ${bufpin_in}:${bufpin_out} -bits
+EOF
+   endif
+   if ($?inbuffers) then
+       cat >> ${modulename}.ys << EOF
+# Input buffering
+iopadmap -inpad ${bufcell} ${bufpin_out}:${bufpin_in} -bits
 EOF
    endif
 endif
@@ -502,11 +645,27 @@ echo set gndnet=\"${gndnet}\" >> ${synthdir}/${modulename}_powerground
 # Run ypostproc syntax post-processor
 #----------------------------------------------------------------------
 
+if (! $?postproc_options) then
+   set postproc_options=""
+else
+   if ( $postproc_options == "-anchors" ) then
+      if ( $?antennacell ) then
+          echo "Running getantennacell to determine cell to use for anchoring." |& tee -a ${synthlog}
+          echo "getantennacell.tcl $modulename ${lefpath} $antennacell" |& tee -a ${synthlog}
+          set anchorinfo=`${scriptdir}/getantennacell.tcl $modulename ${lefpath} $antennacell  | grep antenna= | cut -d= -f2`
+          set postproc_options = "-anchor=$anchorinfo"
+      else
+          echo "Antenna anchoring requested but no antenna cell defined in tech."
+          set postproc_options = ""
+      endif
+   endif
+endif
+
 echo "Cleaning up output syntax" |& tee -a ${synthlog}
-echo "ypostproc.tcl ${modulename}_mapped.blif ${modulename} ${techdir}/${techname}.sh" \
+echo "ypostproc.tcl ${modulename}_mapped.blif ${modulename} ${techdir}/${techname}.sh ${vddnet} ${gndnet} ${postproc_options}" \
 	|& tee -a ${synthlog}
 ${scriptdir}/ypostproc.tcl ${modulename}_mapped.blif ${modulename} \
-		${techdir}/${techname}.sh ${vddnet} ${gndnet}
+		${techdir}/${techname}.sh ${vddnet} ${gndnet} ${postproc_options}
 set errcond = ${status}
 if ( $errcond != 0 ) then
    echo "ypostproc failure.  See file ${synthlog} for error messages." \
@@ -616,15 +775,34 @@ else
       else
 	 set sepoption="-s ${separator}"
       endif
-      if ("x${bufcell}" == "x") then
-	 set bufoption=""
+      if ($?clkbufcell) then
+	 if ("x${clkbufcell}" == "x") then
+	    if ("x${bufcell}" == "x") then
+	       set bufoption=""
+	    else
+	       set bufoption="-b ${bufcell} -i ${bufpin_in} -o ${bufpin_out}"
+	    endif
+	 else
+	    if ("x${bufcell}" == "x") then
+	       set bufoption="-b ${clkbufcell} -i ${bufpin_in} -o ${bufpin_out}"
+	    else
+	       set bufoption="-b ${bufcell},${clkbufcell} -i ${bufpin_in},${clkbufpin_in} -o ${bufpin_out},${clkbufpin_out}"
+	    endif
+	 endif
       else
-	 set bufoption="-b ${bufcell} -i ${bufpin_in} -o ${bufpin_out}"
+	 if ("x${bufcell}" == "x") then
+	    set bufoption=""
+	 else
+	    set bufoption="-b ${bufcell} -i ${bufpin_in} -o ${bufpin_out}"
+	 endif
       endif
 
       # Add paths to liberty files for any hard macros
       set libertyoption="-p ${libertypath}"
       if ( ${?hard_macros} ) then
+         # Do not use the standard cell separator on hard macros!
+	 # Option "-S" sets the separator to null.
+	 set libertyoption="${libertyoption} -S"
 	 foreach macro_path ( $hard_macros )
 	    foreach file ( `ls ${sourcedir}/${macro_path}` )
 	       if ( ${file:e} == "lib" ) then
@@ -636,7 +814,7 @@ else
 
       echo "Running blifFanout (iterative)" |& tee -a ${synthlog}
       echo "Each iteration calls:" |& tee -a ${synthlog}
-      echo "blifFanout ${fanout_options} -I ${modulename}_nofanout ${libertyoption} ${sepoption} ${bufoption} tmp.blif ${modulename}.blif" |& tee -a ${synthlog}
+      echo "blifFanout ${fanout_options} -I ${modulename}_nofanout ${sepoption} ${libertyoption} ${bufoption} tmp.blif ${modulename}.blif" |& tee -a ${synthlog}
       echo "" >> ${synthlog}
 
       set nchanged=1000
@@ -683,6 +861,7 @@ echo "	 ${synthdir}" |& tee -a ${synthlog}
 echo "Files:" |& tee -a ${synthlog}
 echo "   Verilog: ${synthdir}/${modulename}.rtl.v" |& tee -a ${synthlog}
 echo "   Verilog: ${synthdir}/${modulename}.rtlnopwr.v" |& tee -a ${synthlog}
+echo "   Verilog: ${synthdir}/${modulename}.rtlbb.v" |& tee -a ${synthlog}
 echo "   Spice:   ${synthdir}/${modulename}.spc" |& tee -a ${synthlog}
 echo "" >> ${synthlog}
 
@@ -690,9 +869,14 @@ echo "Running blif2Verilog." |& tee -a ${synthlog}
 ${bindir}/blif2Verilog -c -v ${vddnet} -g ${gndnet} ${modulename}.blif \
 	> ${modulename}.rtl.v
 
+# Version without explicit power and ground
 ${bindir}/blif2Verilog -c -p -v ${vddnet} -g ${gndnet} ${modulename}.blif \
 	> ${modulename}.rtlnopwr.v
 
+# Version without vectors
+${bindir}/blif2Verilog -c -p -b -n -v ${vddnet} -g ${gndnet} ${modulename}.blif \
+	> ${modulename}.rtlbb.v
+
 #---------------------------------------------------------------------
 # Spot check:  Did blif2Verilog exit with an error?
 # Note that these files are not critical to the main synthesis flow,
@@ -711,6 +895,12 @@ if ( !( -f ${modulename}.rtlnopwr.v || \
                 |& tee -a ${synthlog}
 endif
 
+if ( !( -f ${modulename}.rtlbb.v || \
+        ( -M ${modulename}.rtlbb.v < -M ${modulename}.blif ))) then
+   echo "blif2Verilog failure:  No file ${modulename}.rtlbb.v created." \
+                |& tee -a ${synthlog}
+endif
+
 #---------------------------------------------------------------------
 
 echo "Running blif2BSpice." |& tee -a ${synthlog}
@@ -737,10 +927,8 @@ else
    set test=`which python3`
    if ( $status == 0 ) then
        echo "Running spi2xspice.py" |& tee -a ${synthlog}
-       if ("x${spicefile}" == "x") then
-           set spiceopt=""
-       else
-           set spiceopt="-l ${spicepath}"
+       if (!( ${?xspice_options} )) then
+           set xspice_options=""
        endif
 
        # Add paths to liberty files for any hard macros
@@ -755,11 +943,11 @@ else
 	  end
        endif
 
-       echo spi2xspice.py '"'${libertyoption}'"' ${modulename}.spc ${modulename}.xspice |& tee -a ${synthlog}
+       echo spi2xspice.py '"'${libertyoption}'"' ${xspice_options} ${modulename}.spc ${modulename}.xspice |& tee -a ${synthlog}
        echo "" >> ${synthlog}
 
-       ${scriptdir}/spi2xspice.py "${libertyoption}" ${modulename}.spc \
-		${modulename}.xspice
+       ${scriptdir}/spi2xspice.py "${libertyoption}" ${xspice_options} \
+		${modulename}.spc ${modulename}.xspice
 
        if ( !( -f ${modulename}.xspice || \
 		( -M ${modulename}.xspice < -M ${modulename}.spc ))) then
