@@ -43,7 +43,11 @@
  *	the swap_group capability of the graywolf placement tool to find
  *	the optimal groupings.
  *
- * Update 6/15/208:
+ * Update 5/7/2018:
+ *	Separated clock buffers from other buffers.  Identify clock inputs
+ *	on flops and latches and trace back to a common clock pin or pins.
+ *
+ * Update 6/15/2018:
  *	Finally got around to doing dynamic string allocation on the input,
  *	which avoids issues of having I/O lines that are arbitrarily long
  *	crashing the program.
@@ -76,17 +80,17 @@
 char *Inputfname;
 char *Outputfname;
 char *Buffername = NULL;
+char *Clkbufname = NULL;
 char *buf_in_pin = NULL;
+char *clkbuf_in_pin = NULL;
 char *buf_out_pin = NULL;
-char *Gatepath = NULL;
+char *clkbuf_out_pin = NULL;
 char *Ignorepath = NULL;
-char *Separator = NULL;
 char SuffixIsNumeric;
 int  GatePrintFlag = 0;
 int  NodePrintFlag = 0;
 int  VerboseFlag = 0;
 int  skip_eol = 0;
-static char default_sep = '\0';
 
 int  Topfanout = 0;
 int  Inputfanout = 0;
@@ -114,6 +118,7 @@ struct Gatelist {
    char   *gatename;
    Cell	  *gatecell;		
    char   *suffix;		// points to position in gatename, not allocated
+   char	  *separator;
    int	  num_inputs;
    double Cint;
    double delay;
@@ -128,6 +133,7 @@ struct Nodelist {
    struct Gatelist *outputgate;
    double outputgatestrength;
    int    type;
+   int    clock;
    int    num_inputs;
    double total_load;
    double ratio;                // drive strength to total_load ratio
@@ -139,7 +145,8 @@ struct Nodelist {
 struct hashlist *Nodehash[OBJHASHSIZE];
 
 struct Drivelist {
-   char *DriveType;	// Suffix name (e.g., "X1")
+   char *Separator;	// Separator (e.g., "X")
+   char *DriveType;	// Suffix name (e.g., "1")
    int NgatesIn;	// Number of gates with this suffix in input
    int NgatesOut;	// Number of gates with this suffix in output
 } Drivelist_;
@@ -154,11 +161,11 @@ struct Baselist {
 
 struct hashlist *Basehash[OBJHASHSIZE];
 
-enum states_ {NONE, INPUTS, OUTPUTS, GATENAME, PINNAME, INPUTNODE, OUTPUTNODE,
-	ENDMODEL, ERROR};
-enum nodetype_ {UNKNOWN, INPUT, OUTPUT, INPUTPIN, OUTPUTPIN};
+enum states_ {NONE, INPUTS, OUTPUTS, GATENAME, PINNAME, INPUTNODE, CLOCKNODE,
+	OUTPUTNODE, ENDMODEL, ERROR};
+enum nodetype_ {UNKNOWN, INPUT, CLOCK, OUTPUT, INPUTPIN, OUTPUTPIN};
 
-int read_gate_file(char *gate_file_name);
+int read_gate_file(char *gate_file_name, char *separator);
 void read_ignore_file(char *ignore_file_name);
 struct Gatelist *GatelistAlloc();
 struct Nodelist *NodelistAlloc();
@@ -177,29 +184,35 @@ void count_gatetype(struct Gatelist *gl, int num_in, int num_out);
  * find_suffix ---
  *
  *	Given a gate name, return the part of the name corresponding to the
- *	suffix.  That is, find the last occurrance of the string Separator
- *	(global variable), and return a pointer to the following character.
+ *	suffix.  That is, find the last occurrance of the string separator
+ *	and return a pointer to the following character.  Note that a NULL
+ *	separator means there is no suffix, vs. an emptry string separator,
+ *	which means that the suffix encompasses all digits at the end of
+ *	the gate name.
  *---------------------------------------------------------------------------
  */
 
-char *find_suffix(char *gatename)
+char *find_suffix(char *gatename, char *separator)
 {
    char *tsuf, *gptr;
    char *suffix = NULL;
 
-   if (Separator == NULL || (*Separator == '\0')) {
+   if (separator == NULL) {
+      return NULL;
+   }
+   else if (*separator == '\0') {
       suffix = gatename + strlen(gatename) - 1;
       while (isdigit(*suffix)) suffix--;
       suffix++;
    }
    else {
       gptr = gatename;
-      while ((tsuf = strstr(gptr, Separator)) != NULL) {
+      while ((tsuf = strstr(gptr, separator)) != NULL) {
          suffix = tsuf;
          gptr = tsuf + 1;
       }
       if (suffix != NULL)
-	 suffix += strlen(Separator);
+	 suffix += strlen(separator);
    }
    return suffix;
 }
@@ -234,6 +247,11 @@ int is_buffer_func(char *func_text, char *pin_in, char *pin_out) {
    return 0;
 }
 
+typedef struct _gaterec {
+    char *path;			/* Path to library */
+    char *sep;			/* Gate name separator ("-" if none) */
+} GateRec;
+
 /*
  *---------------------------------------------------------------------------
  *---------------------------------------------------------------------------
@@ -241,26 +259,28 @@ int is_buffer_func(char *func_text, char *pin_in, char *pin_out) {
 
 int main (int argc, char *argv[])
 {
-   int i, j, k;
+   int i, j, k, l;
    int state;
    int maxline, curline;
+   int libcount;
    int inputcount;
    int gateinputs;
    int gatecount;
    int doLoadBalance = 1;
    int doFanout = 1;
    char *pinname;
-   char *libfile;
+   char *libfile, *libsep;
+   char *separg = NULL;
    char *test;
-   char *s, *t;
+   char *s, *t, *comptr;
    FILE *infptr, *outfptr;
    char *line;
+   GateRec *Gatepaths = NULL;
    struct Gatelist *gl = NULL;
    struct Nodelist *nl = NULL, *nlmax, *nlimax;
    struct Drivelist *dl = NULL;
 
    SuffixIsNumeric = TRUE;	// By default, assume numeric suffixes
-   Separator = NULL;		// By default, assume no separator
 
    /* Note:  To-Do, have an option that sets the case-insensitive */
    /* hash & match functions.					  */
@@ -271,31 +291,51 @@ int main (int argc, char *argv[])
    InitializeHashTable(Nodehash);
    InitializeHashTable(Drivehash);
    InitializeHashTable(Gatehash);
+   InitializeHashTable(Basehash);
 
    fprintf(stdout, "blifFanout for qflow " QFLOW_VERSION "." QFLOW_REVISION "\n");
 
-   while ((i = getopt(argc, argv, "fLgnhvl:c:b:i:o:p:s:I:F:")) != EOF) {
+   while ((i = getopt(argc, argv, "fLSgnhvl:c:b:i:o:p:s:I:F:")) != EOF) {
       switch (i) {
 	 case 'b':
+	    /* If value is a comma-separated pair, the first is a	*/
+	    /* general purpose buffer and the second is a clock buffer.	*/
 	    Buffername = strdup(optarg);
+	    if ((comptr = strchr(Buffername, ',')) != NULL) {
+	       *comptr = '\0';
+	       Clkbufname = comptr + 1;
+	    }
 	    break;
 	 case 'i':
 	    buf_in_pin = strdup(optarg);
+	    if ((comptr = strchr(buf_in_pin, ',')) != NULL) {
+	       *comptr = '\0';
+	       clkbuf_in_pin = comptr + 1;
+	    }
 	    break;
 	 case 'o':
 	    buf_out_pin = strdup(optarg);
+	    if ((comptr = strchr(buf_out_pin, ',')) != NULL) {
+	       *comptr = '\0';
+	       clkbuf_out_pin = comptr + 1;
+	    }
 	    break;
          case 'p':
 	    /* Allow multiple files to be specified as space-separated	*/
 	    /* list, either by specifying all on one "-p" arguments	*/
 	    /* or by passing multiple "-p" arguments.			*/
-	    if (Gatepath == NULL) {
-		Gatepath = strdup(optarg);
+
+	    if (Gatepaths == NULL) {
+		libcount = 1;
+		Gatepaths = (GateRec *)malloc(sizeof(GateRec));
+		Gatepaths->path = strdup(optarg);
+		Gatepaths->sep = (separg) ? strdup(separg) : NULL;
 	    }
 	    else {
-		Gatepath = (char *)realloc(Gatepath, strlen(optarg) +
-			strlen(Gatepath) + 2);
-		sprintf(Gatepath, "%s %s", Gatepath, optarg);
+		libcount++;
+		Gatepaths = (GateRec *)realloc(Gatepaths, libcount * sizeof(GateRec));
+		Gatepaths[libcount - 1].path = strdup(optarg);
+		Gatepaths[libcount - 1].sep = (separg) ? strdup(separg) : NULL;
 	    }
 	    break;
 	 case 'f':	// fanout only
@@ -317,7 +357,16 @@ int main (int argc, char *argv[])
 	    MaxOutputCap = atof(optarg);
 	    break;
 	 case 's':
-	    Separator = strdup(optarg);
+	    if (!strcasecmp(optarg, "none")) {
+	       if (separg) free(separg);
+	       separg = NULL;
+	    }
+	    else
+	       separg = strdup(optarg);
+	    break;
+	 case 'S':
+	    if (separg) free(separg);
+	    separg = NULL;
 	    break;
          case 'g':
 	    GatePrintFlag = 1;
@@ -336,6 +385,18 @@ int main (int argc, char *argv[])
 	    break;
       }
    }
+   if (separg) free(separg);
+
+   /* If there is only one set of in and out pins, then assume	*/
+   /* that the pin names apply to both regular and clock	*/
+   /* buffer types.						*/
+
+   if (clkbuf_in_pin == NULL) {
+      clkbuf_in_pin = buf_in_pin;
+   }
+   if (clkbuf_out_pin == NULL) {
+      clkbuf_out_pin = buf_out_pin;
+   }
 
    Inputfname = Outputfname = NULL;
    infptr = stdin;
@@ -360,20 +421,22 @@ int main (int argc, char *argv[])
    i++;
 
    // Make sure we have a valid gate file path
-   if (Gatepath == NULL) {
-      fprintf(stderr, "blifFanout: No liberty file specified.\n");
+   if (Gatepaths == NULL) {
+      fprintf(stderr, "blifFanout: No liberty file(s) specified.\n");
       return 1;
    }
-   libfile = strtok(Gatepath, " ");
    gatecount = 0;
-   while (libfile != NULL) {
-      gatecount += read_gate_file(libfile);
-      libfile = strtok(NULL, " ");
+   for (l = 0; l < libcount; l++) {
+      int loccount;
+      libfile = Gatepaths[l].path;
+      libsep = Gatepaths[l].sep;
+      loccount = read_gate_file(libfile, libsep);
+      if (loccount == 0)
+         fprintf(stderr, "blifFanout:  Warning:  No gates found in file %s!\n",
+		libfile);
+      gatecount += loccount;
    }
 
-   // Make sure we have a non-NULL separator.
-   if (Separator == NULL) Separator = &default_sep;
-
    // Determine if suffix is numeric or alphabetic
    if (gatecount > 0) {
       char *suffix;
@@ -385,7 +448,7 @@ int main (int argc, char *argv[])
    }
 
    if (gatecount == 0) {
-      fprintf(stderr, "blifFanout:  No gates found in %s file!\n", Gatepath);
+      fprintf(stderr, "blifFanout:  No gates found in any input file!\n");
       return 1;
    }
    if (GatePrintFlag) {
@@ -469,6 +532,9 @@ int main (int argc, char *argv[])
       }
    }
 
+   /* If Clkbufname is not defined, make it the same as Buffername */
+   if (Clkbufname == NULL) Clkbufname = Buffername;
+
    pinname = (char *)malloc(1);
    state = NONE;
    maxline = MAXLINE;
@@ -529,6 +595,8 @@ int main (int argc, char *argv[])
 		      state = OUTPUTNODE;
 		   else if (cur_pintype == PIN_INPUT)
 		      state = INPUTNODE;
+		   else if (cur_pintype == PIN_CLOCK)
+		      state = CLOCKNODE;
 		   else
 		      state = ERROR;	// Probably want error handling here. . .
 	       }
@@ -540,6 +608,12 @@ int main (int argc, char *argv[])
 	       state = PINNAME;
 	       break;
 
+	    case CLOCKNODE:
+	       if (VerboseFlag) printf("\nClock node %s", t);
+	       registernode(t, CLOCK, gl, pinname);
+	       state = PINNAME;
+	       break;
+
 	    case OUTPUTNODE:
 	       if (VerboseFlag) printf("\nOutput node %s", t);
 	       registernode(t, OUTPUT, gl, pinname);
@@ -675,7 +749,7 @@ int main (int argc, char *argv[])
    while (dl != NULL) {
       if (dl->NgatesIn > 0) {
 	 fprintf(stderr, "\t\"%s%s\" gates\tIn: %d    \tOut: %d    \t%+d\n",
-		Separator, dl->DriveType, dl->NgatesIn,
+		dl->Separator, dl->DriveType, dl->NgatesIn,
 		dl->NgatesOut, (dl->NgatesOut - dl->NgatesIn));
       }
       dl = (struct Drivelist *)HashNext(Drivehash);
@@ -738,7 +812,7 @@ void read_ignore_file(char *ignore_file_name)
  *---------------------------------------------------------------------------
  */
 
-int read_gate_file(char *gate_file_name)
+int read_gate_file(char *gate_file_name, char *separator)
 {
    int i, j, k, ind, format = -1;
    int gatecount;
@@ -751,18 +825,19 @@ int read_gate_file(char *gate_file_name)
    gatecount = 0;
    cells = read_liberty(gate_file_name, NULL);
    for (curcell = cells; curcell != NULL; curcell = curcell->next) {
-	if (curcell->name == NULL) continue;	/* "dont_use" cell */
+	if (curcell->name == NULL) continue;	/* undefined/unused cell */
 
 	gl = GatelistAlloc();
 	gl->gatename = strdup(curcell->name);
-	gl->suffix = find_suffix(gl->gatename);
+	gl->suffix = find_suffix(gl->gatename, separator);
+	gl->separator = separator;
 	gl->gatecell = curcell;
 
 	get_values(curcell, &gl->delay, &gl->Cint);
 
 	gl->num_inputs = 0;
 	for (curpin = curcell->pins; curpin; curpin = curpin->next)
-	    if (curpin->type == PIN_INPUT)
+	    if (curpin->type == PIN_INPUT || curpin->type == PIN_CLOCK)
 		gl->num_inputs++;
 
 	/* The "MaxLatency" is empirically derived.  Since gl->delay	*/
@@ -774,14 +849,15 @@ int read_gate_file(char *gate_file_name)
 	/* delays in each gate).					*/
 
 	// (Diagnostic, for debug)
-	// fprintf(stderr, "Parsing cell \"%s\", \"%s\", function \"%s\"\n",
+	// fprintf(stdout, "Parsing cell \"%s\", \"%s\", function \"%s\"\n",
 	//	gl->gatename, gl->gatecell->name, gl->gatecell->function);
 
 	gl->strength = MaxLatency / gl->delay;
  	HashPtrInstall(gl->gatename, gl, Gatehash);
 	gatecount++;
 
-	/* Install prefix in Basehash */
+	/* Install prefix in Basehash.  Note that prefix contains the	*/
+	/* separator string, if any.					*/
 
 	if ((s = gl->suffix) == NULL)
 	    ind = strlen(gl->gatename);
@@ -823,7 +899,7 @@ struct Gatelist* GatelistAlloc()
 
    gl = (struct Gatelist*)malloc(sizeof(struct Gatelist));
    gl->gatename = NULL;
-   gl->suffix = 0;
+   gl->suffix = NULL;
    gl->num_inputs = 0;
    gl->Cint = 0.0;
    gl->delay = 0.0;
@@ -844,6 +920,7 @@ struct Drivelist *DrivelistAlloc()
    dl->NgatesIn = 0;
    dl->NgatesOut = 0;
    dl->DriveType = NULL;
+   dl->Separator = NULL;
    return dl;
 }
 
@@ -866,6 +943,7 @@ struct Nodelist* NodelistAlloc()
    nl->num_inputs = 0;
    nl->num_buf = 0;		// Tree expansion of node
    nl->curcount = 0;
+   nl->clock = FALSE;
    return nl;
 }
 
@@ -906,7 +984,7 @@ void showgatelist(void)
      
       curcell = gl->gatecell;
       for (curpin = curcell->pins; curpin; curpin = curpin->next) {
-	 if (curpin->type == INPUT) {
+	 if (curpin->type == PIN_INPUT || curpin->type == PIN_CLOCK) {
 	    get_pincap(curcell, curpin->name, &pincap);
 	    printf("%g   ", pincap);
  	 }
@@ -943,15 +1021,16 @@ void registernode(char *nodename, int type, struct Gatelist *gl, char *pinname)
 	 count_gatetype(gl, 1, 1);
       }
    }
-   else if (type == INPUT) {
+   else if (type == INPUT || type == CLOCK) {
       if (gl != NULL) {
 	 get_pincap(gl->gatecell, pinname, &pincap);
 	 nl->total_load += pincap;
 	 nl->num_inputs++;
       }
    }
+   if (type == CLOCK) nl->clock = TRUE;
 
-   if ((nl->type != OUTPUTPIN) && (gl == NULL)) {
+   if ((nl->type != INPUTPIN) && (nl->type != OUTPUTPIN) && (gl == NULL)) {
       fprintf(stderr, "\nError: no output gate for net %s\n", nodename);
       fflush(stderr);
    }
@@ -979,6 +1058,7 @@ void count_gatetype(struct Gatelist *gl, int num_in, int num_out)
       dl = DrivelistAlloc();
       HashPtrInstall(s, dl, Drivehash);
       dl->DriveType = strdup(s);
+      dl->Separator = gl->separator;
    }
 
    dl->NgatesIn += num_in;	// Number of these gates before processing
@@ -1025,6 +1105,7 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
    int  gateinputs;
    int  pincount;
    int  needscorrecting;
+   int  hasended;
    int slen;
    int hier;
    char *spos;
@@ -1040,6 +1121,7 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
    state = NONE;
    needscorrecting = 0;
    firstseen = 0;
+   hasended = 0;
 
    // Find the gate record corresponding to the buffer name
    glbuf = (struct Gatelist *)HashLookup(Buffername, Gatehash);
@@ -1087,11 +1169,11 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 			      /* recast "[X]_bF$bufN" as [X_bF$bufN]"	*/
 			      sprintf(spos, "_bF$buf%d]", i);
 			   }
-			   else if (*spos == '>') {
+			   else if (*spos == ']') {
 			      /* Avoid downstream problems:		*/
-			      /* recast "<X>_bF$bufN" as _X_bF$bufN"	*/
+			      /* recast "[X]_bF$bufN" as _X_bF$bufN"	*/
 			      char *dptr = nodename + slen - 1;
-			      while (dptr >= nodename && *dptr != '<') dptr--;
+			      while (dptr >= nodename && *dptr != '[') dptr--;
 			      if (dptr >= nodename) *dptr = '_';
 			      sprintf(spos, "_bF$buf%d", i);
 			   }
@@ -1113,7 +1195,13 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 			      hier++;
 			   }
 			}
-			fprintf(outfptr, ".gate %s %s=%s %s=%s\n",
+			if (nl->clock == TRUE)
+			   fprintf(outfptr, ".gate %s %s=%s %s=%s\n",
+					Clkbufname, clkbuf_in_pin,
+					nl->nodename, clkbuf_out_pin,
+					nodename);
+			else
+			   fprintf(outfptr, ".gate %s %s=%s %s=%s\n",
 					Buffername, buf_in_pin,
 					nl->nodename, buf_out_pin,
 					nodename);
@@ -1124,9 +1212,8 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 	       }
 	       gl = (struct Gatelist *)HashLookup(t, Gatehash);
 	       if (gl == NULL) {
-		  fprintf(stderr, "Fatal error:  Gate \"%s\" is used in source "
+		  fprintf(stderr, "Error:  Gate \"%s\" is used in source "
 			"but has no liberty file definition.\n", t);
-		  fprintf(stderr, "Check if gate is designated \"dont_use\".\n");
 		  gateinputs = 0;
 	       }
 	       else
@@ -1151,12 +1238,16 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 		     state = INPUTNODE;
 		     pincount++;
 		  }
+		  else if (cur_pintype == PIN_CLOCK) {
+		     state = CLOCKNODE;
+		     pincount++;
+		  }
 		  else
 		     state = ERROR;	// Probably want error handling here. . .
 	       }
 	       break;
 
-	    case INPUTNODE:
+	    case INPUTNODE: case CLOCKNODE:
 	       if (VerboseFlag) printf("\nInput node %s", t);
 	       nl = (struct Nodelist *)HashLookup(t, Nodehash);
 	       if (nl->num_buf > 0) {
@@ -1171,11 +1262,11 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 		        /* recast "[X]_bF$bufN" as [X_bF$bufN]"	*/
 		        sprintf(spos, "_bF$buf%d]", nl->curcount);
 		     }
-		     else if (*spos == '>') {
+		     else if (*spos == ']') {
 		        /* Avoid downstream problems:		*/
-		        /* recast "<X>_bF$bufN" as _X_bF$bufN"	*/
+		        /* recast "[X]_bF$bufN" as _X_bF$bufN"	*/
 		        char *dptr = nodename + slen - 1;
-		        while (dptr >= nodename && *dptr != '<') dptr--;
+		        while (dptr >= nodename && *dptr != '[') dptr--;
 		        if (dptr >= nodename) *dptr = '_';
 		        sprintf(spos, "_bF$buf%d", nl->curcount);
 		     }
@@ -1255,6 +1346,8 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 	       if (t[0] == '#') skip_eol = 1;
 	       if (!strcmp(t, ".gate"))
 		  state = GATENAME;
+	       else if (!strcmp(t, ".end"))
+		  state = ENDMODEL;		
 	       break;
 	 }
 	 if (skip_eol == 1) {
@@ -1327,10 +1420,11 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 		     int slen = strlen(glbest->gatename);
 
 		     if (glen != slen) {
-			// The size of the gatename increased, so reallocate
+			// The size of the gatename increased, so reallocate.
+			// Allow additional space for inserted buffer suffix.
 			if (slen > glen) {
 			    gateline = (char *)realloc(gateline, strlen(gateline)
-					+ 1 + slen - glen);
+					+ 10 + slen - glen);
 			    s = strstr(gateline, gl->gatename);
 			}
 			memmove(s + slen, s + glen, strlen(s + glen) + 1);
@@ -1357,7 +1451,10 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 	    gateline[0] = '\0';		/* Starting a new gate */
 
 	    if (state == ENDMODEL)
-	       fprintf(outfptr, "%s", inputline); 
+	       if (hasended == 0) {
+		  fprintf(outfptr, "%s", inputline); 
+		  hasended = 1;
+	       }
          }
 	 else if (state == NONE) {
 	    /* Print line and reset so we don't overflow gateline */
@@ -1365,10 +1462,12 @@ void write_output(int doLoadBalance, FILE *infptr, FILE *outfptr)
 	    gateline[0] = '\0';		/* Start a new line */
 	 }
       }
-      /* Append input line to gate */
-      gateline = (char *)realloc(gateline, strlen(gateline) + strlen(inputline) + 1);
+      /* Append input line to gate.  Always allow additional space for adding	*/
+      /* suffixes to inserted buffers (see above).				*/
+      gateline = (char *)realloc(gateline, strlen(gateline) + strlen(inputline) + 10);
       strcat(gateline, inputline);	/* Append input line to gate */
    }
+   if (hasended == 0) fprintf(outfptr, ".end\n");
    if (VerboseFlag) printf("\n");
    fflush(stdout);
 }
