@@ -215,7 +215,7 @@ xor_expand(char *lib_func)
 	  while (*sptr != ' ' && *sptr != '\t' && *sptr != '\0' &&
 			*sptr != ')')
 	     sptr++;
-	  if (*sptr == ')') sptr--;
+	  if (sptr != fptr) sptr--;
        }
 
        // If argument is a single character, then don't bother with parentheses
@@ -251,10 +251,10 @@ xor_expand(char *lib_func)
 	  }
        }
        else {
-	  while (*sptr != ' ' && *sptr != '\t' && sptr != lib_func &&
+	  while (*sptr != ' ' && *sptr != '\t' && sptr != newfunc &&
 			*sptr != '(')
 	     sptr--;
-	  if (*sptr == '(') sptr++;
+	  if (sptr != newfunc) sptr++;
        }
 
        // If argument is a single character, then don't bother with parentheses
@@ -275,8 +275,12 @@ xor_expand(char *lib_func)
        strcpy(savfunc, newfunc);
        start = savfunc + (sptr - newfunc);
 
-       sprintf(start, "(%s*!%s + !%s*%s) %s",
-		lhs, rhs, lhs, rhs, rest);
+       if (strlen(rest) == 0)
+	  sprintf(start, "(%s*!%s + !%s*%s)",
+			lhs, rhs, lhs, rhs);
+       else
+	  sprintf(start, "(%s*!%s + !%s*%s) %s",
+			lhs, rhs, lhs, rhs, rest);
 
 	if (rhs != NULL) free(rhs);
 	if (lhs != NULL) free(lhs);
@@ -438,6 +442,66 @@ pattern_match(char *name, char *pattern)
     return rval;
 }
 
+/*--------------------------------------------------------------*/
+/* Expand a bus into individual pins				*/
+/*								*/
+/* Return the last pin generated.				*/
+/*--------------------------------------------------------------*/
+
+Pin *
+expand_buses(Pin *curpin, BUStype *curbus, char *busformat)
+{
+    Pin *newpin;
+    char *rootname;
+    int low, high, i;
+    char busbit[1024];
+
+    rootname = curpin->name;
+
+    if (curbus == NULL) {
+	fprintf(stderr, "Error:  Pin %s is a bus, but no bus definition exists!\n",
+		curpin->name);
+	return curpin;
+    }
+    if (curbus->from > curbus->to) {
+	low = curbus->to;
+	high = curbus->from;
+    }
+    else {
+	low = curbus->from;
+	high = curbus->to;
+    }
+
+    /* To do:  Rigorous parsing of busformat.  Assumes that it contains */
+    /* a "%s" followed by "%d".  This is true of all bus formats I can	*/
+    /* think of.  However, that does not mean that it may not have a	*/
+    /* syntax error.							*/
+
+    for (i = low; i <= high; i++) {
+	sprintf(busbit, busformat, rootname, i);
+
+	if (i == low)
+	    newpin = curpin;
+	else
+	    newpin = (Pin *)malloc(sizeof(Pin));
+	newpin->name = strdup(busbit);
+	newpin->next = NULL;
+	if (i != low) {
+	    curpin->next = newpin;
+
+	    /* Copy all pin properties from the old pin to the new pin */
+	    newpin->type = curpin->type;
+	    newpin->cap = curpin->cap;
+	    newpin->maxtrans = curpin->maxtrans;
+	    newpin->maxcap = curpin->maxcap;
+
+	    curpin = newpin;
+	}
+    } 
+    free(rootname);
+    return newpin;
+}
+
 /*--------------------------------------------------------------*/
 /* Read the liberty file and generate the cell database		*/
 /* If "pattern" is non-NULL, then use the pattern to filter the	*/
@@ -452,7 +516,7 @@ read_liberty(char *libfile, char *pattern)
     char *libname = NULL;
     int section = INIT;
     LUTable *tables = NULL;
-    BUStype *buses = NULL;
+    BUStype *buses = NULL, *curbus = NULL;
     Cell *cells = NULL;
 
     double time_unit = 1.0;	// Time unit multiplier, to get ps
@@ -463,7 +527,7 @@ read_liberty(char *libfile, char *pattern)
     char *iptr;
 
     LUTable *newtable, *reftable, *scalar;
-    BUStype *newbus, *refbus;
+    BUStype *newbus;
     Cell *newcell, *lastcell;
     Pin *newpin, *lastpin;
     char *curfunc;
@@ -493,6 +557,10 @@ read_liberty(char *libfile, char *pattern)
     scalar->next = NULL;
     tables = scalar;
 
+    /* Set default bus format (verilog style) */
+
+    busformat = strdup("%s[%d]");
+
     /* Read the file.  This is not a rigorous parser! */
 
     libCurrentLine = 0;
@@ -511,7 +579,8 @@ read_liberty(char *libfile, char *pattern)
 			fprintf(stderr, "Library not followed by name\n");
 		    else
 			token = advancetoken(flib, ')');
-		    fprintf(stderr, "Parsing library \"%s\"\n", token);
+		    /* Diagnostic */
+		    fprintf(stdout, "Parsing library \"%s\"\n", token);
 		    libname = strdup(token);
 		    token = advancetoken(flib, 0);
 		    if (strcmp(token, "{")) {
@@ -544,7 +613,8 @@ read_liberty(char *libfile, char *pattern)
 			exit(1);
 		    }
 		}
-		else if (!strcasecmp(token, "lu_table_template")) {
+		else if (!strcasecmp(token, "lu_table_template") ||
+			!strcasecmp(token, "power_lut_template")) {
 		    // Read in template information;
 		    newtable = (LUTable *)malloc(sizeof(LUTable));
 		    newtable->var1 = NULL;
@@ -707,7 +777,6 @@ read_liberty(char *libfile, char *pattern)
 		    if (strcmp(token, "{"))
 			fprintf(stderr, "Error: failed to find start of block\n");
 		    newcell->reftable = NULL;
-		    newcell->bustype = NULL;
 		    newcell->function = NULL;
 		    newcell->pins = NULL;
 		    newcell->area = 1.0;
@@ -820,6 +889,7 @@ read_liberty(char *libfile, char *pattern)
 		      token = advancetoken(flib, '\"');
 		      if (token == NULL) break;
 		   }
+		   free(busformat);
 		   busformat = strdup(token);
 		   token = advancetoken(flib, ';');
 		}
@@ -875,11 +945,14 @@ read_liberty(char *libfile, char *pattern)
 		break;
 
 	    case CELLDEF:
+		/* Note: "dont_use" is only handled in the case that
+		 * "pattern" is non-NULL;  this is a largely deprecated use.
+		 */
 
 		if (!strcmp(token, "}")) {
 		    section = LIBBLOCK;			// End of cell def
 		}
-		else if (!strcasecmp(token, "dont_use")) {
+		else if (!strcasecmp(token, "dont_use") && (pattern != NULL)) {
 		    token = advancetoken(flib, 0);	// Colon
 		    token = advancetoken(flib, ';');	// To end-of-statement
 		    if (!strcasecmp(token, "true")) {
@@ -897,23 +970,31 @@ read_liberty(char *libfile, char *pattern)
 		    token = advancetoken(flib, 0);	// Open parens
 		    if (!strcmp(token, "("))
 			token = advancetoken(flib, ')');	// Close parens
-		    newpin = (Pin *)malloc(sizeof(Pin));
-		    newpin->name = strdup(token);
 
-		    newpin->next = NULL;
-		    if (lastpin != NULL)
-			lastpin->next = newpin;
-		    else
-			newcell->pins = newpin;
-		    lastpin = newpin;
+		    /* Allow the liberty file to define a pin more than once.	*/
+		    /* This may be done to redefine properties of one pin in a	*/
+		    /* bus, for example.					*/
+
+		    newpin = get_pin_by_name(newcell, token);
+		    if (newpin == NULL) {
+			newpin = (Pin *)malloc(sizeof(Pin));
+			newpin->name = strdup(token);
+			newpin->next = NULL;
+		
+			if (lastpin != NULL)
+			    lastpin->next = newpin;
+			else
+			    newcell->pins = newpin;
+			lastpin = newpin;
+			newpin->type = PIN_UNKNOWN;
+			newpin->cap = 0.0;
+			newpin->maxcap = 0.0;
+			newpin->maxtrans = 0.0;
+		    }
 
 		    token = advancetoken(flib, 0);	// Find start of block
 		    if (strcmp(token, "{"))
 			fprintf(stderr, "Error: failed to find start of block\n");
-		    newpin->type = PIN_UNKNOWN;
-		    newpin->cap = 0.0;
-		    newpin->maxcap = 0.0;
-		    newpin->maxtrans = 0.0;
 		    section = PINDEF;
 		}		
 		else if (!strcasecmp(token, "area")) {
@@ -942,6 +1023,9 @@ read_liberty(char *libfile, char *pattern)
 	    case PINDEF:
 
 		if (!strcmp(token, "}")) {
+		    if (curbus != NULL)
+			lastpin = expand_buses(lastpin, curbus, busformat);
+		    curbus = NULL;
 		    section = CELLDEF;			// End of pin def
 		}
 		else if (!strcasecmp(token, "capacitance")) {
@@ -971,26 +1055,35 @@ read_liberty(char *libfile, char *pattern)
 		    token = advancetoken(flib, 0);	// Colon
 		    token = advancetoken(flib, ';');
 		    /* Find the bus */
-		    for (refbus = buses; refbus; refbus = refbus->next)
-			if (!strcmp(refbus->name, token))
+		    for (curbus = buses; curbus; curbus = curbus->next)
+			if (!strcmp(curbus->name, token))
 			    break;
-		    if (refbus == NULL)
+		    if (curbus == NULL)
 			fprintf(stderr, "Failed to find a valid bus type \"%s\"\n",
 				token);
-		    else if (newcell->bustype == NULL) {
-			newcell->bustype = refbus;
-		    }
 		}
 		else if (!strcasecmp(token, "direction")) {
 		    token = advancetoken(flib, 0);	// Colon
 		    token = advancetoken(flib, ';');
 		    if (!strcasecmp(token, "input")) {
-			newpin->type = PIN_INPUT;
+			if (newpin->type != PIN_CLOCK)
+			   newpin->type = PIN_INPUT;
 		    }
 		    else if (!strcasecmp(token, "output")) {
 			newpin->type = PIN_OUTPUT;
 		    }
 		}
+		else if (!strcasecmp(token, "clock")) {
+		    token = advancetoken(flib, 0);	// Colon
+		    token = advancetoken(flib, ';');
+		    if (!strcasecmp(token, "true")) {
+			if (newpin->type == PIN_INPUT || newpin->type == PIN_UNKNOWN)
+			   newpin->type = PIN_CLOCK;
+			else {
+			   fprintf(stderr, "Warning: Output pin defined as clock.\n");
+			}
+		    }
+		}
 		else if (!strcasecmp(token, "max_transition")) {
 		    token = advancetoken(flib, 0);	// Colon
 		    token = advancetoken(flib, ';');	// To end-of-statement
@@ -1034,6 +1127,7 @@ read_liberty(char *libfile, char *pattern)
 
 	    case TIMING:
 
+		/* To do:  Record delays separately for each pin, or average */
 		if (!strcmp(token, "}")) {
 		    section = PINDEF;			// End of timing def
 		}
@@ -1093,8 +1187,13 @@ read_liberty(char *libfile, char *pattern)
 				while ((iptr = strchr(iptr, ',')) != NULL) {
 				    iptr++;
 				    i++;
-				    sscanf(iptr, "%lg", &newcell->times[i]);
-				    newcell->times[i] *= time_unit;
+				    if (i < reftable->tsize) {
+					sscanf(iptr, "%lg", &newcell->times[i]);
+					newcell->times[i] *= time_unit;
+				    } else {
+					fprintf(stderr, "Too many entries for table"
+						" (line %d).\n", libCurrentLine);
+				    }
 				}
 			    }
 
@@ -1137,8 +1236,14 @@ read_liberty(char *libfile, char *pattern)
 				while ((iptr = strchr(iptr, ',')) != NULL) {
 				    iptr++;
 				    i++;
-				    sscanf(iptr, "%lg", &newcell->caps[i]);
-				    newcell->caps[i] *= cap_unit;
+				    if (i < reftable->csize) {
+					sscanf(iptr, "%lg", &newcell->caps[i]);
+					newcell->caps[i] *= cap_unit;
+				    }
+				    else {
+					fprintf(stderr, "Too many entries for table"
+						" (line %d).\n", libCurrentLine);
+				    }
 				}
 			    }
 
@@ -1314,7 +1419,7 @@ get_pincap(Cell *curcell, char *pinname, double *retcap)
 
     for (curpin = curcell->pins; curpin; curpin = curpin->next) {
 	if (!strcmp(curpin->name, pinname)) {
-	    if (curpin->type == PIN_INPUT) {
+	    if (curpin->type == PIN_INPUT || curpin->type == PIN_CLOCK) {
 		*retcap = curpin->cap;
 		return 0;
 	    }
@@ -1330,32 +1435,6 @@ get_pincap(Cell *curcell, char *pinname, double *retcap)
 
 /*--------------------------------------------------------------------*/
 
-int
-get_pintype(Cell *curcell, char *pinname)
-{
-    Pin *curpin;
-    char *buschar = NULL;
-
-    /* Is pin part of a bus?  If so, return the root pin */
-    if (curcell->bustype != NULL) {
-	/* Not checking bounds or syntax;  blif requires <...> */
-	buschar = strrchr(pinname, '<');
-	if (buschar != NULL) *buschar = '\0';
-    }
-
-    for (curpin = curcell->pins; curpin; curpin = curpin->next) {
-	if (!strcmp(curpin->name, pinname)) {
-	    if (buschar != NULL) *buschar = '<';
-	    return curpin->type;
-	}
-    }
-
-    if (buschar != NULL) *buschar = '<';
-    return PIN_UNKNOWN;
-}
-
-/*--------------------------------------------------------------------*/
-
 Cell *
 get_cell_by_name(Cell *cell, char *name)
 {
@@ -1387,6 +1466,8 @@ get_cell_by_name(Cell *cell, char *name)
     return NULL;
 }
 
+/*--------------------------------------------------------------------*/
+
 Pin *
 get_pin_by_name(Cell *curcell, char *pinname)
 {
@@ -1398,10 +1479,25 @@ get_pin_by_name(Cell *curcell, char *pinname)
             return curpin;
         }
     }
-    printf("did not find pin %s\n", pinname);
     return NULL;
 }
 
+/*--------------------------------------------------------------------*/
+
+int
+get_pintype(Cell *curcell, char *pinname)
+{
+    Pin *curpin = get_pin_by_name(curcell, pinname);
+    if (curpin != NULL)
+	return curpin->type;
+    else {
+	printf("Error: did not find pin %s\n", pinname);
+	return PIN_UNKNOWN;
+    }
+}
+
+/*--------------------------------------------------------------------*/
+
 void delete_Cell(Cell *cell) {
     Pin *curpin = cell->pins;
     Pin *tmppin;
